/*
 Copyright (C) 2013 Marco Vassallo <gedeone-octave@users.sourceforge.net>

 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free Software
 Foundation; either version 3 of the License, or (at your option) any later
 version.

 This program is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 details.

 You should have received a copy of the GNU General Public License along with
 this program; if not, see <http://www.gnu.org/licenses/>.
*/
#include "functionspace.h"

DEFUN_DLD (SubSpace, args, , "-*- texinfo -*-\n\
@deftypefn {Function File} {[@var{V1}]} = \
SubSpace (@var{V}, @var{index}) \n\
Extract a SubSpace from an object of type FunctionSpace. \n\
The input arguments are\n\
@itemize @bullet\n\
@item @var{V} which is a FunctionalSpace\n\
@item @var{index} is a positive integer number which represents the SubSpace \
which has to be extracted.\n\
@end itemize \n\
The output @var{V1} is the SubSpace needed.\n\
@seealso{FunctionSpace}\n\
@end deftypefn")
{
  int nargin = args.length ();
  octave_value retval;

  if (nargin < 2 || nargin > 2)
    print_usage ();
  else
    {
      if (! functionspace_type_loaded)
        {
          functionspace::register_type ();
          functionspace_type_loaded = true;
          mlock ();
        }

      if (args(0).type_id () == functionspace::static_type_id ())
        {
          const functionspace & fspo
            = static_cast<const functionspace&> (args(0).get_rep ());
          const octave_idx_type idx = args(1).int_value ();

          if (! error_state)
            {
              const dolfin::FunctionSpace & V = fspo.get_fsp ();

              if (idx < 1 || idx > V.dim ())
                error ("SubSpace: index out of bounds");

              else
                {
                  boost::shared_ptr <const dolfin::FunctionSpace>
                    g (new dolfin::SubSpace (V, idx - 1));

                  retval = new functionspace (g);
                }

            }
        }
    }
  return retval;
}
