/*
 Copyright (C) 2013 Marco Vassallo <gedeone-octave@users.sourceforge.net>

 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free Software
 Foundation; either version 3 of the License, or (at your option) any later
 version.

 This program is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 details.

 You should have received a copy of the GNU General Public License along with
 this program; if not, see <http://www.gnu.org/licenses/>.
*/

#include "function.h"

DEFUN_DLD (feval, args, , "-*- texinfo -*-\n\
@deftypefn {Function File} {[@var{value}]} = \
feval (@var{function_name}, @var{Coordinate})\n\
Evaluate a function at a specific point of the domain and return the value. \n\
The input parameters are the function and the point where it has to\
be evaluated.\n\
@seealso{Function}\n\
@end deftypefn")
{

  int nargin = args.length ();
  octave_value retval=0;
  
  if (nargin < 2 || nargin > 2)
    print_usage ();
  else
    {
      if (! function_type_loaded)
        {
          function::register_type ();
          function_type_loaded = true;
          mlock ();
        }

      if (args(0).type_id () == function::static_type_id ())
        {
          const function & fspo =
            static_cast<const function&> (args(0).get_rep ());
          Array<double> point= args(1).array_value ();

          if (!error_state)
            {
              const boost::shared_ptr<const dolfin::Function> 
                & f = fspo.get_pfun ();
              dim_vector dims;
              dims.resize (2);
              dims(0) = 1;
              dims(1) = f->value_dimension (0);
              Array<double> res (dims);
              dolfin::Array<double> x(point.length (), point.fortran_vec ());
              dolfin::Array<double> values(res.length (), res.fortran_vec ());

              f->eval (values, x);

              retval = octave_value (res);
            }
        }
    }
  return retval;
}
