/*
 Copyright (C) 2013 Marco Vassallo <gedeone-octave@users.sourceforge.net>

 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free Software
 Foundation; either version 3 of the License, or (at your option) any later
 version.

 This program is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 details.

 You should have received a copy of the GNU General Public License along with
 this program; if not, see <http://www.gnu.org/licenses/>.
*/

#ifndef _FUNCTION_OCTAVE_
#define _FUNCTION_OCTAVE_

#include <memory>
#include <vector>
#include <dolfin.h>
#include <octave/oct.h>

class function : public octave_base_value
{

 public:

  function ()
    : octave_base_value (), fun () {}

  function (std::string & _str, 
            boost::shared_ptr <const dolfin::Function> _fun)
    : octave_base_value (), str(_str), fun (_fun) {}

  void 
  print (std::ostream& os,
         bool pr_as_read_syntax = false) const
    { os << "Function " << str << ": " << fun->str (true) << std::endl; }

  ~function(void) {}

  bool 
  is_defined (void) const
    { return true; }

  const dolfin::Function &
  get_fun (void) const
    { return (*fun); }

  const boost::shared_ptr <const dolfin::Function> &
  get_pfun (void) const
    { return fun; }

  void 
  set_fun (dolfin::Function & _fun)
    {
      dolfin::Function * p = new dolfin::Function (_fun);
      fun = boost::shared_ptr<const dolfin::Function> (p);
    }

  const std::string & 
  get_str (void) const
    { return str; }

 private:

  std::string str;
  boost::shared_ptr <const dolfin::Function> fun;

  DECLARE_OCTAVE_ALLOCATOR;
  DECLARE_OV_TYPEID_FUNCTIONS_AND_DATA;

};
static bool function_type_loaded = false;

DEFINE_OCTAVE_ALLOCATOR (function);
DEFINE_OV_TYPEID_FUNCTIONS_AND_DATA (function, "function", "function");
#endif
